"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const os_1 = require("os");
const ts_util_is_1 = require("ts-util-is");
const helpers_1 = require("../common/helpers");
/**
 * MSSQL generator.
 */
class MSSQLGenerator {
    constructor(config) {
        this.config = config;
    }
    /**
     * Current configuration.
     */
    config;
    /**
     * Get data file content.
     *
     * @param item Row from query.
     */
    data(item, sett) {
        let output = '';
        switch (sett.idempotency.data) {
            case 'delete':
                output += `DELETE FROM [${item.schema}].[${item.name}]` + os_1.EOL;
                output += os_1.EOL;
                break;
            case 'delete-and-reseed':
                output += `DELETE FROM [${item.schema}].[${item.name}]`;
                output += os_1.EOL;
                output += `DBCC CHECKIDENT ('[${item.schema}].[${item.name}]', RESEED, 0)`;
                output += os_1.EOL;
                break;
            case 'truncate':
                output += `TRUNCATE TABLE [${item.schema}].[${item.name}]`;
                output += os_1.EOL;
                break;
        }
        output += os_1.EOL;
        if (item.hasIdentity) {
            output += `SET IDENTITY_INSERT [${item.schema}].[${item.name}] ON`;
            output += os_1.EOL;
            output += os_1.EOL;
        }
        item.result.recordset.forEach((row) => {
            const keys = Object.keys(row);
            const columns = keys.join(', ');
            const values = keys.map((key) => this.safeValue(row[key])).join(', ');
            output += `INSERT INTO [${item.schema}].[${item.name}] (${columns}) VALUES (${values})`;
            output += os_1.EOL;
        });
        if (item.hasIdentity) {
            output += os_1.EOL;
            output += `SET IDENTITY_INSERT [${item.schema}].[${item.name}] OFF`;
            output += os_1.EOL;
        }
        output += os_1.EOL;
        return output;
    }
    /**
     * Get Upgrade Audit header.
     *
     * @param version current software version.
     * @param type script types.
     */
    upgradeAudit(name, version, type) {
        let output = '';
        output += `-- ************************** UPGRADE AUDIT ********************************`;
        output += os_1.EOL;
        output += `INSERT INTO UpgradeAudit (ScriptName) VALUES ('${name} Upgrade ${version} - 3 ${type}.sql')`;
        output += os_1.EOL;
        output += `GO`;
        output += os_1.EOL;
        output += `-- *************************************************************************`;
        return output;
    }
    /**
     * Get function file content.
     *
     * @param item Row from query.
     */
    function(item, sett) {
        const objectId = `[${item.schema}].[${item.name}]`;
        const type = item.type.trim();
        let output = '';
        switch (sett.idempotency.functions) {
            case 'if-exists-drop':
                output += `IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID('${objectId}') AND type = '${type}')`;
                output += os_1.EOL;
                output += `DROP FUNCTION ${objectId}`;
                output += os_1.EOL;
                output += 'GO';
                output += os_1.EOL;
                break;
            case 'if-not-exists':
                output += `IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID('${objectId}') AND type = '${type}')`;
                output += os_1.EOL;
                break;
        }
        output += item.text;
        return output;
    }
    /**
     * Get stored procedure file content.
     *
     * @param item Row from query.
     */
    storedProcedure(item, sett) {
        const objectId = `[${item.schema}].[${item.name}]`;
        const type = item.type.trim();
        let output = '';
        switch (sett.idempotency.procs) {
            case 'if-exists-drop':
                output += `IF EXISTS (`;
                output += os_1.EOL;
                output += `   SELECT 1`;
                output += os_1.EOL;
                output += `   FROM sys.objects`;
                output += os_1.EOL;
                output += `   WHERE object_id = OBJECT_ID('${objectId}')`;
                output += os_1.EOL;
                output += `     AND type = '${type}'`;
                output += os_1.EOL;
                output += `   )`;
                output += os_1.EOL;
                output += ` DROP PROCEDURE ${objectId}`;
                output += os_1.EOL;
                output += 'GO';
                output += os_1.EOL;
                output += os_1.EOL;
                output += `SET ANSI_NULLS ON`;
                output += os_1.EOL;
                output += `GO`;
                output += os_1.EOL;
                output += os_1.EOL;
                output += `SET QUOTED_IDENTIFIER ON`;
                output += os_1.EOL;
                output += `GO`;
                output += os_1.EOL;
                output += os_1.EOL;
                break;
            case 'if-not-exists':
                output += `IF NOT EXISTS (`;
                output += os_1.EOL;
                output += `   SELECT 1`;
                output += os_1.EOL;
                output += `   FROM sys.objects`;
                output += os_1.EOL;
                output += `   WHERE object_id = OBJECT_ID('${objectId}')`;
                output += os_1.EOL;
                output += `     AND type = '${type}'`;
                output += os_1.EOL;
                output += `   )`;
                output += os_1.EOL;
                break;
        }
        output += this.formatSQL(item.text);
        return output;
    }
    formatSQL(text) {
        text = text.trim();
        text = text.replaceAll('create procedure', 'CREATE PROCEDURE');
        text = text.replaceAll('set nocount on', 'SET NOCOUNT ON');
        text = text.replaceAll('select', 'SELECT');
        text = text.replaceAll('distinct', 'DISTINCT');
        text = text.replaceAll('month', 'MONTH');
        text = text.replaceAll('year', 'YEAR');
        text = text.replaceAll('datename', 'DATENAME');
        text = text.replaceAll('from', 'FROM');
        text = text.replaceAll('begin', 'BEGIN');
        text = text.replaceAll('update', 'UPDATE');
        text = text.replaceAll('set', 'SET');
        text = text.replaceAll('declare', 'DECLARE');
        text = text.replaceAll('if ', 'IF ');
        text = text.replaceAll(' as ', ' AS ');
        text = text.replaceAll(' on ', ' ON ');
        text = text.replaceAll('where', 'WHERE');
        text = text.replaceAll('case', 'CASE');
        text = text.replaceAll('when', 'WHEN');
        text = text.replaceAll('then', 'THEN');
        text = text.replaceAll('else', 'ELSE');
        text = text.replaceAll('exists', 'EXISTS');
        text = text.replaceAll('count', 'COUNT');
        text = text.replaceAll('cast', 'CAST');
        text = text.replaceAll('convert', 'CONVERT');
        text = text.replaceAll('inner join', 'INNER JOIN');
        text = text.replaceAll('left join', 'LEFT JOIN');
        text = text.replaceAll('right join', 'RIGHT JOIN');
        text = text.replaceAll('outer join', 'OUTER JOIN');
        text = text.replaceAll('order', 'ORDER');
        text = text.replaceAll('group', 'GROUP');
        text = text.replaceAll(' by ', ' BY ');
        text = text.replaceAll('and ', 'AND ');
        text = text.replaceAll(' or ', ' OR ');
        text = text.replaceAll('isnull', 'ISNULL');
        text = text.replaceAll('is null', 'IS NULL');
        text = text.replaceAll('is not null', 'IS NOT NULL');
        text = text.replaceAll('int', 'INT');
        text = text.replaceAll('varchar', 'VARCHAR');
        text = text.replaceAll(' bit ', ' BIT ');
        text = text.replaceAll('datetime', 'DATETIME');
        return text;
    }
    permissions(item, name) {
        let output = os_1.EOL;
        output += os_1.EOL;
        for (let i = 0; i < item.length; i++) {
            const element = item[i];
            output += `GRANT EXECUTE`;
            output += os_1.EOL;
            output += ` ON[${name}]`;
            output += os_1.EOL;
            output += ` TO[${element.principal_name}]`;
            output += os_1.EOL;
            output += `GO`;
            output += os_1.EOL;
            output += os_1.EOL;
        }
        return output;
    }
    /**
     * Get schema file content.
     *
     * @param item Row from query.
     */
    schema(item) {
        let output = '';
        output += `IF NOT EXISTS (SELECT 1 FROM sys.schemas WHERE name = '${item.name}')`;
        output += os_1.EOL;
        output += `EXEC('CREATE SCHEMA ${item.name}')`;
        return output;
    }
    /**
     * Get table file content.
     *
     * @param item Row from query.
     * @param columns Columns from query.
     * @param primaryKeys Primary key from query.
     * @param foreignKeys Foreign keys from query.
     * @param indexes Indexes from query.
     */
    table(item, columns, primaryKeys, foreignKeys, indexes, sett) {
        const objectId = `[${item.schema}].[${item.name}]`;
        const type = item.type.trim();
        let output = '';
        switch (sett.idempotency.tables) {
            case 'if-exists-drop':
                output += `IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID('${objectId}') AND type = '${type}')`;
                output += os_1.EOL;
                output += `DROP TABLE ${objectId}`;
                output += os_1.EOL;
                output += 'GO';
                output += os_1.EOL;
                break;
            case 'if-not-exists':
                output += `IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID('${objectId}') AND type = '${type}')`;
                output += os_1.EOL;
                break;
        }
        output += `CREATE TABLE ${objectId}`;
        output += os_1.EOL;
        output += '(';
        output += os_1.EOL;
        columns
            .filter((x) => x.object_id === item.object_id)
            .forEach((col) => {
            output += this.indent() + this.column(col, sett) + ',';
            output += os_1.EOL;
        });
        primaryKeys = primaryKeys.filter((x) => x.object_id === item.object_id);
        foreignKeys = foreignKeys.filter((x) => x.object_id === item.object_id);
        indexes = indexes.filter((x) => x.object_id === item.object_id);
        const groupedKeys = helpers_1.Helpers.groupByName(primaryKeys, 'name');
        Object.keys(groupedKeys).forEach((name) => {
            output += this.primaryKey(groupedKeys[name]);
            output += os_1.EOL;
        });
        output += ')';
        if (foreignKeys.length || indexes.length) {
            output += os_1.EOL;
            output += os_1.EOL;
        }
        foreignKeys.forEach((fk) => {
            output += this.foreignKey(fk);
            output += os_1.EOL;
        });
        const groupedIndexes = helpers_1.Helpers.groupByName(indexes, 'name');
        Object.keys(groupedIndexes).forEach((name) => {
            output += this.index(groupedIndexes[name]);
            output += os_1.EOL;
        });
        return output;
    }
    /**
     * Get trigger file content.
     *
     * @param item Row from query.
     */
    trigger(item, sett) {
        const objectId = `[${item.schema}].[${item.name}]`;
        const type = item.type.trim();
        let output = '';
        switch (sett.idempotency.triggers) {
            case 'if-exists-drop':
                output += `IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID('${objectId}') AND type = '${type}')`;
                output += os_1.EOL;
                output += `DROP TRIGGER ${objectId}`;
                output += os_1.EOL;
                output += 'GO';
                output += os_1.EOL;
                break;
            case 'if-not-exists':
                output += `IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID('${objectId}') AND type = '${type}')`;
                output += os_1.EOL;
                break;
        }
        output += item.text;
        return output;
    }
    /**
     * Get type file content.
     *
     * @param item Row from query.
     */
    type(item, sett) {
        const objectId = `[${item.schema}].[${item.name}]`;
        let output = '';
        switch (sett.idempotency.types) {
            case 'if-exists-drop':
                output += 'IF EXISTS (';
                output += os_1.EOL;
                output += this.indent() + 'SELECT 1 FROM sys.types AS t';
                output += os_1.EOL;
                output +=
                    this.indent() + 'JOIN sys.schemas s ON t.schema_id = s.schema_id';
                output += os_1.EOL;
                output +=
                    this.indent() +
                        `WHERE t.name = '${item.name}' AND s.name = '${item.schema}'`;
                output += os_1.EOL;
                output += ')';
                output += os_1.EOL;
                output += `DROP TYPE ${item.name}`;
                output += os_1.EOL;
                output += 'GO';
                output += os_1.EOL;
                break;
            case 'if-not-exists':
                output += `IF NOT EXISTS (`;
                output += os_1.EOL;
                output += this.indent() + 'SELECT 1 FROM sys.types AS t';
                output += os_1.EOL;
                output +=
                    this.indent() + 'JOIN sys.schemas s ON t.schema_id = s.schema_id';
                output += os_1.EOL;
                output +=
                    this.indent() +
                        `WHERE t.name = '${item.name}' AND s.name = '${item.schema}'`;
                output += os_1.EOL;
                output += ')';
                output += os_1.EOL;
                break;
        }
        output += `CREATE TYPE ${objectId}`;
        output += os_1.EOL;
        output += `FROM ${item.system_type.toUpperCase()}`;
        switch (item.system_type) {
            case 'char':
            case 'nvarchar':
            case 'varchar':
                output += `(${item.max_length === -1 ? 'max' : item.max_length})`;
                break;
            case 'decimal':
            case 'numeric':
                output += `(${item.scale},${item.precision})`;
                break;
        }
        output += item.is_nullable ? ' NULL' : ' NOT NULL';
        return output;
    }
    /**
     * Get table type file content.
     *
     * @param item Row from query.
     * @param columns Columns from query.
     */
    tableType(item, columns, sett) {
        const objectId = `[${item.schema}].[${item.name}]`;
        const type = item.type.trim();
        let output = '';
        switch (sett.idempotency.types) {
            case 'if-exists-drop':
                output += 'IF EXISTS (';
                output += os_1.EOL;
                output += this.indent() + 'SELECT 1 FROM sys.table_types AS t';
                output += os_1.EOL;
                output +=
                    this.indent() + 'JOIN sys.schemas s ON t.schema_id = s.schema_id';
                output += os_1.EOL;
                output +=
                    this.indent() +
                        `WHERE t.name = '${item.name}' AND s.name = '${item.schema}'`;
                output += os_1.EOL;
                output += ')';
                output += os_1.EOL;
                output += `DROP TYPE ${objectId}`;
                output += os_1.EOL;
                output += 'GO';
                output += os_1.EOL;
                break;
            case 'if-not-exists':
                output += `IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID('${objectId}') AND type = '${type}')`;
                output += os_1.EOL;
                break;
        }
        output += `CREATE TYPE ${objectId} AS TABLE`;
        output += os_1.EOL;
        output += '(';
        output += os_1.EOL;
        columns
            .filter((x) => x.object_id === item.object_id)
            .forEach((col, idx, array) => {
            output += this.indent() + this.column(col, sett);
            if (idx !== array.length - 1) {
                // not the last column
                output += ',';
            }
            output += os_1.EOL;
        });
        output += ')';
        return output;
    }
    /**
     * Get view file content.
     *
     * @param item Row from query.
     */
    view(item, sett) {
        const objectId = `[${item.schema}].[${item.name}]`;
        const type = item.type.trim();
        let output = '';
        switch (sett.idempotency.views) {
            case 'if-exists-drop':
                output += `IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID('${objectId}') AND type = '${type}')`;
                output += os_1.EOL;
                output += `DROP VIEW ${objectId}`;
                output += os_1.EOL;
                output += 'GO';
                output += os_1.EOL;
                break;
            case 'if-not-exists':
                output += `IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID('${objectId}') AND type = '${type}')`;
                output += os_1.EOL;
                break;
        }
        output += item.text;
        return output;
    }
    /**
     * Get job file content.
     *
     * @param steps Steps from query.
     * @param schedules Schedules from query.
     */
    job(job, steps, schedules, sett) {
        let output = '';
        switch (sett.idempotency.views) {
            case 'if-exists-drop':
                output += `IF EXISTS (SELECT 1 FROM msdb.dbo.sysjobs WHERE name = '${job.name}')`;
                output += os_1.EOL;
                output += `EXEC msdb.dbo.sp_delete_job @job_name = '${job.name}'`;
                output += os_1.EOL;
                output += 'GO';
                output += os_1.EOL;
                output += os_1.EOL;
                output += this.addJob(job, steps, schedules);
                output += os_1.EOL;
                break;
            case 'if-not-exists':
                output += `IF NOT EXISTS (SELECT 1 FROM msdb.dbo.sysjobs WHERE name = '${job.name}')`;
                output += os_1.EOL;
                output += 'BEGIN';
                output += os_1.EOL;
                output += this.addJob(job, steps, schedules);
                output += os_1.EOL;
                output += 'END';
                output += os_1.EOL;
                break;
        }
        return output;
    }
    /**
     * Safely transform SQL value for scripting.
     *
     * @param value SQL data value.
     */
    safeValue(value) {
        if ((0, ts_util_is_1.isNull)(value)) {
            return 'NULL';
        }
        if ((0, ts_util_is_1.isString)(value)) {
            value = value.replace("'", "''");
            return `'${value}'`;
        }
        if ((0, ts_util_is_1.isDate)(value)) {
            value = value.toISOString();
            return `'${value}'`;
        }
        if ((0, ts_util_is_1.isBoolean)(value)) {
            return value ? 1 : 0;
        }
        return value;
    }
    /**
     * Get script for table's column.
     *
     * @param item Row from query.
     */
    column(item, sett) {
        let output = `[${item.name}]`;
        let size;
        if (item.is_computed) {
            output += ` AS ${item.formula}`;
            if (item.is_persisted) {
                output += ' PERSISTED';
                output += item.is_nullable ? ' NULL' : ' NOT NULL';
            }
            return output;
        }
        output += ` ${item.datatype}`;
        switch (item.datatype) {
            case 'varchar':
            case 'char':
            case 'varbinary':
            case 'binary':
            case 'text':
                size = item.max_length === -1 ? 'max' : item.max_length;
                output += `(${size})`;
                break;
            case 'nvarchar':
            case 'nchar':
            case 'ntext':
                size = item.max_length === -1 ? 'max' : item.max_length / 2;
                output += `(${size})`;
                break;
            case 'datetime2':
            case 'time2':
            case 'datetimeoffset':
                output += `(${item.scale})`;
                break;
            case 'decimal':
                output += `(${item.precision}, ${item.scale})`;
                break;
        }
        if (item.collation_name && !item.is_user_defined) {
            output += ` COLLATE ${item.collation_name}`;
        }
        output += item.is_nullable ? ' NULL' : ' NOT NULL';
        if (item.definition) {
            if (sett.includeConstraintName && item.default_name) {
                output += ` CONSTRAINT [${item.default_name}]`;
            }
            output += ` DEFAULT${item.definition}`;
        }
        if (item.is_identity) {
            output += ` IDENTITY(${item.seed_value || 0}, ${item.increment_value || 1})`;
        }
        return output;
    }
    /**
     * Get script for table's primary key.
     *
     * @param items Rows from query.
     */
    primaryKey(items) {
        const first = items[0];
        let output = '';
        output += this.indent() + `CONSTRAINT [${first.name}] PRIMARY KEY `;
        switch (first.type) {
            case 'CLUSTERED':
                output += 'CLUSTERED ';
                break;
            case 'NONCLUSTERED':
                output += 'NONCLUSTERED ';
                break;
        }
        if (items.length > 1) {
            output += os_1.EOL;
            output += this.indent() + '(';
            output += os_1.EOL;
            items.forEach((item, idx, array) => {
                output += this.indent(2) + this.primaryKeyColumn(item);
                if (idx !== array.length - 1) {
                    // not the last column
                    output += ',';
                }
                output += os_1.EOL;
            });
            output += this.indent() + ')';
        }
        else {
            output += '(' + this.primaryKeyColumn(first) + ')';
        }
        return output;
    }
    /**
     * Get script for an individual primary key column.
     *
     * @param item Row from query.
     */
    primaryKeyColumn(item) {
        const direction = item.is_descending_key ? 'DESC' : 'ASC';
        return `[${item.column}] ${direction}`;
    }
    /**
     * Get script for table's foreign key.
     *
     * @param item Row from foreignKeys query.
     */
    foreignKey(item) {
        const objectId = `[${item.schema}].[${item.table}]`;
        const keyObjectId = `[${item.schema}].[${item.name}]`;
        const parentObjectId = `[${item.parent_schema}].[${item.parent_table}]`;
        let output = '';
        output += `IF NOT EXISTS (SELECT 1 FROM sys.foreign_keys WHERE object_id = OBJECT_ID('${keyObjectId}') AND parent_object_id = OBJECT_ID('${objectId}'))`;
        output += os_1.EOL;
        output += 'BEGIN';
        output += os_1.EOL;
        output +=
            this.indent() +
                `ALTER TABLE ${objectId} WITH ${item.is_not_trusted ? 'NOCHECK' : 'CHECK'}`;
        output += ` ADD CONSTRAINT [${item.name}] FOREIGN KEY ([${item.column}])`;
        output += ` REFERENCES ${parentObjectId} ([${item.reference}])`;
        switch (item.delete_referential_action) {
            case 1:
                output += ' ON DELETE CASCADE';
                break;
            case 2:
                output += ' ON DELETE SET NULL';
                break;
            case 3:
                output += ' ON DELETE SET DEFAULT';
                break;
        }
        switch (item.update_referential_action) {
            case 1:
                output += ' ON UPDATE CASCADE';
                break;
            case 2:
                output += ' ON UPDATE SET NULL';
                break;
            case 3:
                output += ' ON UPDATE SET DEFAULT';
                break;
        }
        output += os_1.EOL;
        output +=
            this.indent() + `ALTER TABLE ${objectId} CHECK CONSTRAINT [${item.name}]`;
        output += os_1.EOL;
        output += 'END';
        output += os_1.EOL;
        return output;
    }
    /**
     * Get script for table's index.
     *
     * @param items Rows from query.
     */
    index(items) {
        const first = items[0];
        const objectId = `[${first.schema}].[${first.table}]`;
        let output = '';
        output += `IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE object_id = OBJECT_ID('${objectId}') AND name = '${first.name}')`;
        output += os_1.EOL;
        output += 'CREATE';
        if (first.is_unique) {
            output += ' UNIQUE';
        }
        output += ` ${first.type} INDEX [${first.name}] ON ${objectId}`;
        output += '(';
        if (items.length > 1) {
            output += os_1.EOL;
            items.forEach((item, idx, array) => {
                output += this.indent() + this.indexColumn(item);
                if (idx !== array.length - 1) {
                    // not the last column
                    output += ',';
                }
                output += os_1.EOL;
            });
        }
        else {
            output += this.indexColumn(first);
        }
        output += ')';
        output += os_1.EOL;
        return output;
    }
    /**
     * Get script for an individual index column.
     *
     * @param item Row from query.
     */
    indexColumn(item) {
        const direction = item.is_descending_key ? 'DESC' : 'ASC';
        return `[${item.column}] ${direction}`;
    }
    /**
     * Get job file content.
     *
     * @param steps Steps from query.
     * @param schedules Schedules from query.
     */
    addJob(job, steps, schedules) {
        let output = '';
        // job
        output += 'EXEC msdb.dbo.sp_add_job ';
        output += os_1.EOL;
        output += this.indent() + `@job_name = N'${job.name}',`;
        output += os_1.EOL;
        output += this.indent() + `@enabled = ${job.enabled},`;
        output += os_1.EOL;
        output += this.indent() + `@description = N'${job.description}',`;
        output += os_1.EOL;
        output +=
            this.indent() + `@notify_level_eventlog = ${job.notify_level_eventlog},`;
        output += os_1.EOL;
        output +=
            this.indent() + `@notify_level_email = ${job.notify_level_email},`;
        output += os_1.EOL;
        output +=
            this.indent() + `@notify_level_netsend = ${job.notify_level_netsend},`;
        output += os_1.EOL;
        output += this.indent() + `@notify_level_page = ${job.notify_level_page},`;
        output += os_1.EOL;
        output += this.indent() + `@delete_level = ${job.delete_level};`;
        output += os_1.EOL;
        output += 'GO';
        output += os_1.EOL;
        output += os_1.EOL;
        // steps
        steps.forEach((step) => {
            output += 'EXEC msdb.dbo.sp_add_jobstep';
            output += os_1.EOL;
            output += this.indent() + `@job_name = N'${step.job_name}',`;
            output += os_1.EOL;
            output += this.indent() + `@step_name = N'${step.step_name}',`;
            output += os_1.EOL;
            output += this.indent() + `@subsystem = N'${step.subsystem}',`;
            output += os_1.EOL;
            output += this.indent() + `@command = N'${step.command}',`;
            output += os_1.EOL;
            if (step.additional_parameters) {
                output +=
                    this.indent() +
                        `@additional_parameters = N'${step.additional_parameters}',`;
                output += os_1.EOL;
            }
            output +=
                this.indent() + `@cmdexec_success_code = ${step.cmdexec_success_code},`;
            output += os_1.EOL;
            output +=
                this.indent() + `@on_success_action = ${step.on_success_action},`;
            output += os_1.EOL;
            output +=
                this.indent() + `@on_success_step_id = ${step.on_success_step_id},`;
            output += os_1.EOL;
            output += this.indent() + `@on_fail_action = ${step.on_fail_action},`;
            output += os_1.EOL;
            output += this.indent() + `@on_fail_step_id = ${step.on_fail_step_id},`;
            output += os_1.EOL;
            output += this.indent() + `@database_name = N'${step.database_name}',`;
            output += os_1.EOL;
            if (step.database_user_name) {
                output +=
                    this.indent() +
                        `@database_user_name = N'${step.database_user_name}',`;
                output += os_1.EOL;
            }
            output += this.indent() + `@retry_attempts = ${step.retry_attempts},`;
            output += os_1.EOL;
            output += this.indent() + `@retry_interval = ${step.retry_interval},`;
            output += os_1.EOL;
            output += this.indent() + `@os_run_priority = ${step.os_run_priority},`;
            output += os_1.EOL;
            output += this.indent() + `@flags = ${step.flags};`;
            output += os_1.EOL;
            output += 'GO';
            output += os_1.EOL;
            output += os_1.EOL;
        });
        // schedule
        if (schedules.length) {
            output += 'EXEC msdb.dbo.sp_add_schedule';
            output += os_1.EOL;
            schedules.forEach((schedule) => {
                output +=
                    this.indent() + `@schedule_name = N'${schedule.schedule_name}',`;
                output += os_1.EOL;
                output += this.indent() + `@enabled = ${schedule.enabled},`;
                output += os_1.EOL;
                output += this.indent() + `@freq_type = ${schedule.freq_type},`;
                output += os_1.EOL;
                output += this.indent() + `@freq_interval = ${schedule.freq_interval},`;
                output += os_1.EOL;
                output +=
                    this.indent() + `@freq_subday_type = ${schedule.freq_subday_type},`;
                output += os_1.EOL;
                output +=
                    this.indent() +
                        `@freq_subday_interval = ${schedule.freq_subday_interval},`;
                output += os_1.EOL;
                output +=
                    this.indent() +
                        `@freq_relative_interval = ${schedule.freq_relative_interval},`;
                output += os_1.EOL;
                output +=
                    this.indent() +
                        `@freq_recurrence_factor = ${schedule.freq_recurrence_factor},`;
                output += os_1.EOL;
                output +=
                    this.indent() + `@active_start_date = ${schedule.active_start_date},`;
                output += os_1.EOL;
                output +=
                    this.indent() + `@active_end_date = ${schedule.active_end_date},`;
                output += os_1.EOL;
                output +=
                    this.indent() + `@active_start_time = ${schedule.active_start_time},`;
                output += os_1.EOL;
                output +=
                    this.indent() + `@active_end_time = ${schedule.active_end_time};`;
                output += os_1.EOL;
            });
            output += 'GO';
            output += os_1.EOL;
            output += os_1.EOL;
            // attach
            const scheduleName = schedules[0].schedule_name;
            output += 'EXEC msdb.dbo.sp_attach_schedule';
            output += os_1.EOL;
            output += this.indent() + `@job_name = N'${job.name}',`;
            output += os_1.EOL;
            output += this.indent() + `@schedule_name = N'${scheduleName}';`;
            output += os_1.EOL;
            output += 'GO';
            output += os_1.EOL;
            output += os_1.EOL;
        }
        // job server
        output += 'EXEC msdb.dbo.sp_add_jobserver';
        output += os_1.EOL;
        output += this.indent() + `@job_name = N'${job.name}';`;
        output += os_1.EOL;
        output += 'GO';
        output += os_1.EOL;
        return output;
    }
    /**
     * Generate indentation spacing.
     *
     * @param count Number of levels to indent.
     */
    indent(count = 1) {
        return '    '.repeat(count);
    }
}
exports.default = MSSQLGenerator;
//# sourceMappingURL=mssql.js.map